<?php

$query = (isset($_GET['query']))? $_GET['query'] : '';
$filter = (isset($_GET['filter']))? $_GET['filter'] : 'All';
$newfilter = (isset($_GET['newfilter']))? $_GET['newfilter'] : '';
$global = (isset($_GET['global']))? $_GET['global'] : false;
$action = (isset($_GET['action']))? $_GET['action'] : '';
$simple = (isset($_GET['simple']))? $_GET['simple'] : 0;
$type = (isset($_GET['type']))? $_GET['type'] : false;
$username = (isset($_GET['username']))? $_GET['username'] : false;
$groups = (isset($_GET['groups']))? explode('|', $_GET['groups']) : array();
$format = (isset($_GET['format']))? $_GET['format'] : 'html';
if (count($groups) == 1 && $groups[0] === '')
	$groups[0] = 'All';
else
	array_push($groups, 'All');

$model = new FiltersModel;

if ($action === 'remove' && $filter !== 'All')
{
	Audit::track('remove filter', "Filter: '$filter'");
	if (Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username))
		$model->remove_filter($filter);
	else
		echo '<span class="hidden"></span>'; // error
}
elseif ($action === 'rename' && $filter !== 'All')
{
	Audit::track('rename filter', "Filter: '$filter'");
	if (Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username)
		&& !$model->rename_filter($filter, $newfilter))
	{
		echo ($type)? '<option class="hidden" value=""></option>' : '<span class="hidden"></span>'; // error
	}
}
elseif ($action === 'make_global' && $filter !== 'All')
{
	Audit::track('make filter global', "Filter: '$filter'");
	$model->make_global($filter, Auth::$username);
}
elseif ($action === 'make_own' && $filter !== 'All')
{
	Audit::track('make filter private', "Filter: '$filter'");
	$model->make_own($filter, Auth::$username);
}
elseif ($action === 'check_children')
{
	if ($model->has_children($filter))
		echo '<span class="hidden"></span>'; // the filter has children
}
elseif ($newfilter != '' && $query != '')
{
	// handle a situation where user tries to create a filter based on nothing
	if ($filter === '')
		echo '<span class="hidden"></span>'; // error
	elseif ($model->exists($newfilter))
	{
		Audit::track('edit filter', "Filter: '$newfilter'; Parent: '$filter'");
		if ($model->check_parents($filter, $newfilter))
		{
			// do not allow replacing other users' private filters
			if (Auth::$is_admin['filter'] && ($model->is_own($newfilter, Auth::$username)
				|| $model->is_global($newfilter)))
			{
				$model->update_filter($newfilter, $query, $filter);
				// force new global/own state depending on the checkbox
				if ($global)
					$model->make_global($newfilter, Auth::$username);
				else
					$model->make_own($newfilter, Auth::$username);
			}
			elseif ($model->is_own($newfilter, Auth::$username))
				$model->update_filter($newfilter, $query, $filter);
			else
				echo '<span class="hidden"></span>'; // error
		}
		else
			echo '<span class="hidden"></span>'; // error
	}
	else
	{
		Audit::track('create filter', "Filter: '$newfilter'; Parent: '$filter'");
		if (Auth::$is_admin['filter'] && $global)
			$model->add_global_filter($newfilter, $query, $filter);
		else
			$model->add_own_filter($newfilter, $query, Auth::$username, $filter);
	}
	// note that user can cheat to create a filter based on inaccessible filter
	// (or modify existing to get the same result), but that will not work anyway,
	// so no anti-cheat checks are added here to avoid increasing complexity
}

if ($action == 'get_query')
{
	$query = $model->get_query($filter);
	echo '<input type="text" id="filter_query" name="filter_query" value="'
		. htmlspecialchars($query)
		. '" maxlength="128" readonly="readonly" class="normal" style="color: #A9A9A9;" />';
	exit;
}
elseif ($action == 'get_info' && $filter !== '')
{
	if (Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username))
	{
		$parentfilter = $model->get_parent_filter($filter);
		echo '<span>' . (($parentfilter !== null)? $parentfilter : '') . '</span>';
		echo '<span>' . $model->get_query($filter) . '</span>';
		echo '<span>' . (($model->is_global($filter))? 'true' : 'false') . '</span>';
	}
	exit;
}
elseif ($action == 'get_parents' && $filter !== '')
{
	if (Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username)
		|| $model->is_accessible($filter, Auth::$username))
	{
		if ((Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username)) && $filter != 'All')
			$hierarchy = '<span class="' . ($model->is_global($filter)? 'global' : 'own')
				. '" style="font-size: 11px; font-weight: bold;">' . $filter . '</span>';
		else
			$hierarchy = '<span class="' . ($model->is_global($filter)? 'global' : 'own')
				. '" style="font-size: 11px; font-weight: bold; color: #999999;">' . $filter . '</span>';

		$currentfilter = $filter;

		while (true)
		{
			$parentfilter = $model->get_parent_filter($currentfilter);
			if ($parentfilter !== null)
			{
				$currentfilter = $parentfilter;
				if (Auth::$is_admin['filter'] || $model->is_own($currentfilter, Auth::$username))
					$hierarchy = '<a href="#"' . (($currentfilter != 'All')? 'style="text-decoration: none; border-bottom: 1px dotted #999999;"' : ' style="color: #999999;"')
						. '>' . $currentfilter . '</a> <em style="font-style: normal;">&nbsp;&raquo;&nbsp;</em> ' . $hierarchy;
				elseif ($model->is_accessible($currentfilter, Auth::$username))
					$hierarchy = '<a href="#" style="color: #999999;">' . $currentfilter . '</a> <em style="font-style: normal;">&nbsp;&raquo;&nbsp;</em> ' . $hierarchy;
				else
					break;
			}
			else
				break;
		}

		$hierarchy .= '&nbsp;';
		if ((Auth::$is_admin['filter'] || $model->is_own($filter, Auth::$username)) && $filter != 'All')
		{
			if ($model->get_parent_filter($filter) !== null)
				$hierarchy .= ' <a href="#" id="edit" class="button"><img src="/logserver/images/icons/edit-sml.png" style="margin-right: -4px; margin-bottom: -4px; " /></a>';

			$hierarchy .= ' <a href="#" id="delete" class="button"><img src="/logserver/images/icons/remove-sml.png" style="margin-right: -4px; margin-bottom: -4px; " /></a>';
		}

		if (Auth::$is_admin['filter'])
			$hierarchy .= ' <a href="#" id="permissions" class="button" style="text-decoration: none"><img src="/logserver/images/icons/manage-permissions.png" style="margin-bottom: -4px" title="Manage permissions" /> <em style="display:none;border-bottom: 1px dotted #777777">manage permissions</em></a>';
	}
}
elseif ($action == 'get_visible')
{
	if ($username !== '')
	{
		$global_filters = $model->get_user_filters($username);
		$own_filters = $model->get_own_filters($username);
	}
	else
	{
		$global_filters = array();
		$own_filters = array();
	}

	foreach ($groups as $groupname)
	{
		$group_filters = $model->get_group_filters($groupname);
		$global_filters = array_merge($global_filters, array_diff($group_filters, $global_filters));
	}
	natcasesort($global_filters);
}
elseif ($action == 'get_exportable')
{
	if (Auth::$is_admin['filter'])
	{
		$exportable_filters = $model->get_exportable_filters();
	}
}
else
{
	$global_filters = $model->get_global_filters(Auth::$username);
	$own_filters = $model->get_own_filters(Auth::$username);
	$default_filter = $model->get_default_filter(Auth::$username);
	if (!in_array($default_filter, $global_filters) && !in_array($default_filter, $own_filters))
		unset($default_filter);
}
