<?php

class ImportModel
{

	public function name_to_path($name, $basepath) {
		$path = str_replace("messages_", $basepath . "/", $name);
		$path = preg_replace('/_/', '$', $path, 1);
		$path = str_replace("_", "/", $path);
		$path = str_replace("$", "_", $path);
		return $path.".bz2";
	}

	public function check_size($year, $month, $day = false, $hour = false) {

		/* Get configured data path of local storage and check partition free space*/

		$result = Database::query("SELECT * from settings");
		$result = $result[0];
		$path = $result['local_storage_path'];

		$pg_conf = array();
		$pg_conf_cont = file_get_contents('/etc/init.d/postgresql');
		$pg_conf_cont = explode("\n", $pg_conf_cont);

		foreach ($pg_conf_cont as $conf) {
			$is_conf = (strpos($conf, "=") !== false) ? true : false;
			if ($is_conf) {
				$kvp = explode("=", $conf);
				$pg_conf[$kvp[0]] = $kvp[1];
			}
		}

		$pg_path = explode("/", $pg_conf['PGDATA']);
		array_pop($pg_path);

		$pg_path = implode("/", $pg_path);
		$tspace = disk_free_space($pg_path);

		/* 
			Check tables_registry entries if they are not imported/inmporting get 
			them and if the correlating archives exist, check their sizes 
		*/
		
		if ($hour != false) {
			$batchname = $year."-".$month."-".$day."-".$hour;
			$query = "SELECT * FROM tables_registry WHERE name LIKE 'messages_" . $year."_".$month."_".$day."_".$hour . "%'  AND import_status IS NULL ORDER BY name";
		} elseif ($day != false) {
			$batchname = $year."-".$month."-".$day;
			$query = "SELECT * FROM tables_registry WHERE name LIKE 'messages_" . $year."_".$month."_".$day. "%'  AND import_status IS NULL ORDER BY name";
		} else {
			$batchname = $year."-".$month;
			$query = "SELECT * FROM tables_registry WHERE name LIKE 'messages_" . $year."_".$month . "%'  AND import_status IS NULL ORDER BY name";
		}

		$entries = Database::query($query);
		$size = 0;

		foreach ($entries as $entry) {
			$filepath = self::name_to_path($entry['name'], $path);
			
			if (file_exists($filepath)) {
				$size = $size + filesize($filepath);
			}
		}
		
		$data = array(
			"space" => $tspace,
			"filename" => $batchname,
			"size" => $size,
			"size_mod" => $result['compressed_filesize_modifier'],
			"fail_zone" => $result['min_disk_size_free']
		);

		echo json_encode($data);
		exit;
	}

	public function import_hour($year, $month, $day, $hour)
	{
		Database::query("SELECT change_import_status('messages_{$year}_{$month}_{$day}_{$hour}', 'planned')");
	}

	public function import_day($year, $month, $day)
	{
		Database::query("UPDATE tables_registry SET import_status = 'planned' WHERE name LIKE 'messages_{$year}_{$month}_{$day}_%'"
			. " AND get_location(location) != 'database'"
			. " AND (NOT (import_status = 'started' OR import_status = 'planned') OR import_status IS NULL)");
	}

	public function import_month($year, $month)
	{
		Database::query("UPDATE tables_registry SET import_status = 'planned' WHERE name LIKE 'messages_{$year}_{$month}_%'"
			. " AND get_location(location) != 'database'"
			. " AND (NOT (import_status = 'started' OR import_status = 'planned') OR import_status IS NULL)");
	}

	public function is_in_db($year, $month, $day, $hour)
	{
		$result = Database::query("SELECT get_location(location) AS location FROM tables_registry"
			. " WHERE name = 'messages_{$year}_{$month}_{$day}_{$hour}'");
		return ($result[0]['location'] == 'database')? true : false;
	}

	public function is_importing($year, $month, $day, $hour)
	{
		$result = Database::query("SELECT * FROM tables_registry WHERE name = 'messages_{$year}_{$month}_{$day}_{$hour}'"
			. " AND (import_status = 'planned' OR import_status = 'started')");
		return (count($result))? true : false;
	}

	public function list_tables()
	{
		return Database::query("SELECT SUBSTRING(name FROM 10 FOR 13)"
			. " AS table FROM tables_registry WHERE location != B'000'"
			. " AND (import_status = 'started' OR import_status = 'planned') ORDER BY name");
	}

	public function check_tables($check)
	{
		$tables = array();

		foreach ($check as $table)
		{
			$result = Database::query("SELECT SUBSTRING(name FROM 10 FOR 13)"
				. " AS table, import_status FROM tables_registry WHERE name = 'messages_$table'");
			if (count($result))
				if ($result[0]['import_status'] == 'finished')
					$tables[$result[0]['table']] = true;
				elseif ($result[0]['import_status'] == 'failed')
					$tables[$result[0]['table']] = false;
		}

		return $tables;
	}
}
