<?php

if ($_SERVER['REMOTE_ADDR'] != '127.0.0.1' && !Auth::$username)
{
	header('Location: /logserver/error.html');
	exit;
}

require_once ('comps/filters/model.php');
require_once ('comps/settings/model.php');
require_once ('comps/inquire/lsq2html.php');

$default_limit = 75;

$filter = (isset($_GET['filter']))? $_GET['filter'] : 'All';
$query = (isset($_GET['query']))? $_GET['query'] : '';
$table = (isset($_GET['table']))? $_GET['table'] : '';
$direction = (isset($_GET['direction']))? $_GET['direction'] : 'backwards';
$offset = (isset($_GET['offset']))? $_GET['offset'] : 0;
$limit = (isset($_GET['limit']))? $_GET['limit'] : $default_limit;
$order = (isset($_GET['order']))? $_GET['order'] : 'desc';

$start_time = (isset($_GET['start_time']))? $_GET['start_time'] : '';
$end_time = (isset($_GET['end_time']))? $_GET['end_time'] : '';

$offset_time = (isset($_GET['offset_time']))? $_GET['offset_time'] : '';
$offset_only = isset($_GET['offset_time']);

$lmodel = new LogsModel;
$fmodel = new FiltersModel;

$fullquery = $query;
// note that $query is used in view so don't change the lines above

if (isset($_GET['html']))
{
	if ($query !== '')
	{
		try
		{
			echo LSQ2HTML::translate(LSQParser::parseQuery($query));
		}
		catch(Exception $e)
		{
			echo '';
		}
	}

	exit;
}

$fields = array(
	'rtime',
	'((facsev & 1016) / 8) as fac',
	'(facsev & 7) as sev',
	'mtime',
	'host',
	'ip',
	'ident',
	'pid',
	'event',
	'msg'
);

if (!$offset_only)
	Audit::track('view logs', "Filter: '$filter'; Query: '$query'"
		. (($table == '' && $start_time != '' && $end_time != '')?
		"; From: $start_time; To: $end_time" : ''));

if ($_SERVER['REMOTE_ADDR'] == '127.0.0.1'
	|| Auth::$is_admin['filter']
	|| $fmodel->is_own($filter, Auth::$username)
	|| $fmodel->is_accessible($filter, Auth::$username))
{
	$fullquery = $fmodel->get_query($filter) . ' ' . $fullquery;
	$currentfilter = $filter;
	$error = false;

	if (!Auth::$is_admin['filter'] && $fmodel->is_global($currentfilter))
	{
		if ($fmodel->is_accessible($currentfilter, Auth::$username))
			$first_global_ok = true;
		else
			$first_global_ok = false;
	}

	while (true)
	{
		$parentfilter = $fmodel->get_parent_filter($currentfilter);
		if ($parentfilter === null)
		{
			if ($currentfilter != 'All')
				$error = "The parent of filter '$currentfilter' does not exist.";
			break;
		}

		$currentfilter = $parentfilter;

		if (!Auth::$is_admin['filter'] && $fmodel->is_global($currentfilter)
			&& !isset($first_global_ok))
		{
			if ($fmodel->is_accessible($currentfilter, Auth::$username))
				$first_global_ok = true;
			else
				$first_global_ok = false;
		}

		if ($_SERVER['REMOTE_ADDR'] == '127.0.0.1'
			|| Auth::$is_admin['filter']
			|| $fmodel->is_own($currentfilter, Auth::$username)
			|| ($fmodel->is_global($currentfilter) && $first_global_ok))
		{
			$fullquery = $fmodel->get_query($currentfilter) . ' ' . $fullquery;
		}
		else
		{
			$error = "You have no permissions to access the parent of filter '$currentfilter'.";
			break;
		}
	}

	if (!$error)
	{
		$smodel = new SettingsModel;
		$settings = $smodel->retrieve_settings();

		try
		{
			if ($table == '' && $start_time != '' && $end_time != '')
			{
				$logs = array();
				foreach ($lmodel->tables_in_range($start_time, $end_time) as $table)
					$logs = array_merge($logs, $lmodel->get_logs($fullquery, implode(',', $fields),
						$table['name'], 0, $limit, 'forward', $start_time, $end_time));
			}
			else if ($offset_only)
			{
				echo $lmodel->get_offset($query, $table, $direction, $offset_time);
				exit;
			}
			else
				$logs = $lmodel->get_logs($fullquery, implode(',', $fields), $table,
					abs($offset), $limit, $direction, '', '');
		}
		catch(Exception $e)
		{
			$logs = array();
			$error = "Database query failed!
			Either you have used a regexp ('~' operator) with invalid syntax,
			or the database connection is down.";
			if ($offset_only)
				exit;
		}
	}
}
else
	$error = "You have no permissions to access filter '$filter'.";

