<?php

class LSQ2SQL
{
	protected static $columns = array(
		'ip'        => 'TextType',
		'src_ip'    => 'ip',
		'source_ip' => 'ip',
		'sourceip'  => 'ip',
		'rtime'     => 'TimeType',
		'recv_time' => 'rtime',
		'fac'       => 'FacilityType',
		'facility'  => 'fac',
		'sev'       => 'SeverityType',
		'severity'  => 'sev',
		'mtime'     => 'TimeType',
		'msg_time'  => 'mtime',
		'host'      => 'TextType',
		'ident'     => 'TextType',
		'pid'       => 'PIDType',
		'event'     => 'IntType',
		'event_id'  => 'event',
		'eventid'   => 'event',
		'msg'       => 'TextType',
		'message'   => 'msg',
		''          => 'FullTextType'
	);


	protected static function expression($condition, $column, $operator, $args)
	{
		if (!array_key_exists($column, self::$columns))
			throw new Exception("Unknown column name");

		$type = self::$columns[$column];

		if (array_key_exists($type, self::$columns))
		{
			$column = $type;
			$type = self::$columns[$type];
		}

		if(!class_exists($type))
			throw new Exception("Unknown column type");

		return call_user_func(array($type, $operator), $condition, $column, $args);
	}


	public static function translate($tree)
	{
		$query = array();
		$ftsnodes = array();
		$ftsquery = array();

		foreach ($tree as $node)
			if ($node['column'] !== '')
				$query[] = self::expression($node['condition'], $node['column'], $node['operator'], $node['args']);
			else
				$ftsnodes[] = $node;

		/* ugly hack to work around this:
		select * from messages where to_tsvector('english', fts) @@ to_tsquery(' ( !cron ) ');
		ERROR:  query requires full scan, which is not supported by GIN indexes
		by doing this:
		select * from messages where not to_tsvector('english', fts) @@ to_tsquery(' ( cron ) ');
		*/
		$hack = false;
		foreach ($ftsnodes as $node)
			if ($node['condition'])
			{
				$hack = false;
				break;
			}
			else
				$hack = true;
		if ($hack)
			foreach ($ftsnodes as $index => $node)
				$ftsnodes[$index]['condition'] = true;

		foreach ($ftsnodes as $node)
			$ftsquery[] = self::expression($node['condition'], $node['column'], $node['operator'], $node['args']);

		if (count($ftsnodes) > 0)
			$query[] = (($hack)? 'NOT ' : '') . "to_tsvector('english', fts) @@ to_tsquery('"
				. implode((($hack)? ' | ' : ' & '), $ftsquery) . "')";

		return implode(' AND ', $query);
	}
}
