<?php

class LSQParser
{
	private static $string;
	private static $tree = array();
	private static $previousElement = '';
	private static $operator = '';
	private static $condition = true;

	public static $defaultOperator = '=';

	public static $operators = array(
		":" => "column",
		"=" => "equals",
		"~" => "tilda"
	);


	private static function getElementType()
	{
		$string = &self::$string;

		$char = $string{0};

		if ($char == '-')
		{
			self::$condition = false;
			$string = substr($string, 1);
			$char = $string{0};
		}

		if (in_array($char, array_keys(self::$operators)))
			return 'pair';
		elseif ($char == '(')
			return 'group';
		elseif ($char == '"' || $char == "'")
			return 'string';
		else
			return 'word';
	}


	private static function getPair()
	{
		$string = &self::$string;

		$string = substr($string, 1);
		$element = substr($string, 0, strcspn($string, " (\"'"));
		$string = substr($string, strlen($element));

		return $element;
	}


	private static function getGroup()
	{
		$string = &self::$string;

		$string = substr($string, 1);
		$element = $string;
		$string = substr($string, strlen($element) + 1);

		return $element;
	}


	private static function getString()
	{
		$string = &self::$string;

		$type = $string{0}; /* can be ' or " */
		$string = substr($string, 1);
		$element = substr($string, 0, strcspn($string, $type));
		$string = substr($string, strlen($element) + 1);

		return $element;
	}


	/* see the notice for getString */
	private static function getWord($in_group = false)
	{
		$string = &self::$string;

		/*
		the '"' char is to allow msg"foo bar" construction
		to be equivalent to using the default operator
		remove it from here and from the bottom of parse()
		when this feature is not needed
		*/
		$element = substr($string, 0, strcspn($string, ($in_group)? " )\"'" : " :=~(\"'"));
		$string = substr($string, strlen($element));

		return $element;
	}


	private static function parseGroup()
	{
		$string = &self::$string;

		$tree = array();

		while (true)
		{
			$string = trim($string);
			if (strlen($string) == 0)
				break;

			if ($string{0} == '"' || $string{0} == "'")
				$element = self::getString();
			elseif ($string{0} == ')')
			{
				// remove the ')' after group
				// including any extra ')' insterted by user by mistake
				while ($string{0} == ')')
					$string = substr($string, 1);
				break;
			}
			elseif ($string{0} == '(')
			{
				// skip extra '(' that was inserted by user by mistake
				$string = substr($string, 1);
				continue;
			}
			else
				$element = self::getWord(true);

			$tree[] = $element;
		}

		return $tree;
	}


	public static function addElement($operator, $args)
	{
		self::$tree[] = array(
			'condition' => self::$condition,
			'column' => self::$previousElement,
			'operator' => $operator,
			'args' => $args
		);
		self::$condition = true;
	}


	public static function parseQuery($query)
	{
		$string = &self::$string;
		$string = $query;

		// reset default values to avoid issues when calling the parser more than once
		self::$tree = array();
		self::$previousElement = '';
		self::$operator = self::$defaultOperator;
		self::$condition = true;

		while (true)
		{
			if (self::$previousElement === '')
				$string = trim($string);
			/*
			Note: after an element the string is not trimmed to avoid a situation
			when 'foo= bar' is parsed as 'foo=bar' instead of 'foo="" bar'
			*/

			if (strlen($string) == 0)
				break;

			$type = self::getElementType();
			if ($type == 'pair')
			{
				self::$operator = $string{0};
				$element = self::getPair();
				if ($element !== '')
				{
					self::addElement(self::$operators[self::$operator], array($element));
					self::$previousElement = '';
				}
			}
			elseif ($type == 'group')
			{
				$string = substr($string, 1);
				self::addElement(self::$operators[self::$operator], self::parseGroup());
				self::$previousElement = '';
			}
			elseif ($type == 'string')
			{
				$element = self::getString();
				if (self::$previousElement === '')
				{
					self::$operator = self::$defaultOperator;
					self::addElement(self::$operators[self::$operator], array($element));
				}
				else
				{
					self::addElement(self::$operators[self::$operator], array($element));
					self::$previousElement = '';
				}
			}
			else
			{
				$element = self::getWord();
				if (strlen($string) == 0 || $string{0} === ' ')
				{
					self::$operator = self::$defaultOperator;
					self::addElement(self::$operators[self::$operator], array($element));
					self::$previousElement = '';
				}
				else
				{
					self::$previousElement = $element;
					// allow msg(foo bar) to be equivalent to msg=(foo bar)
					// and msg"foo bar" to be equivalent to msg="foo bar"
					if ($string{0} == '(' || $string{0} == '"' || $string{0} == "'")
						self::$operator = self::$defaultOperator;
				}
			}
		}

		return self::$tree;
	}
}
