<?php

class ColType
{
	public static function equals($condition, $column, $args)
	{
		throw new Exception("Operator = is not supported for $column column");
	}


	public static function column($condition, $column, $args)
	{
		throw new Exception("Operator : is not supported for $column column");
	}


	public static function tilda($condition, $column, $args)
	{
		throw new Exception("Operator ~ is not supported for $column column");
	}
}


class TextType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . (($condition)? '' : ' NOT') . " ILIKE '%" . pg_escape_string($arg) . "%'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}


	public static function column($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . (($condition)? '' : ' NOT') . " ILIKE '" . pg_escape_string($arg) . "%'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}


	public static function tilda($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . (($condition)? ' ' : ' !') . "~ E'" . pg_escape_string($arg) . "'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}


class FacilityType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$fac = array(
			'kernel'   => 0,
			'user'     => 1,
			'mail'     => 2,
			'daemon'   => 3,
			'auth'     => 4,
			'syslog'   => 5,
			'lpr'      => 6,
			'news'     => 7,
			'uucp'     => 8,
			'cron'     => 9,
			'authpriv' => 10,
			'ftp'      => 11,
			'ntp'      => 12,
			'logaudit' => 13,
			'logalert' => 14,
			'clock2'   => 15,
			'local0'   => 16,
			'local1'   => 17,
			'local2'   => 18,
			'local3'   => 19,
			'local4'   => 20,
			'local5'   => 21,
			'local6'   => 22,
			'local7'   => 23,
            'mark'     => 24
		);

		$result = array();

		foreach($args as $arg)
			if (array_key_exists($arg, $fac))
				$result[] = (($condition)? '' : 'NOT ') . '((facsev & 1016) / 8) = ' . $fac[$arg];

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}

class SeverityType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$sev = array(
			'emergency' => 0,
			'emerg'     => 0,
			'alert'     => 1,
			'critical'  => 2,
			'crit'      => 2,
			'error'     => 3,
			'err'       => 3,
			'warning'   => 4,
			'warn'      => 4,
			'notice'    => 5,
			'info'      => 6,
			'debug'     => 7
		);

		$result = array();

		foreach($args as $arg)
			if (array_key_exists($arg, $sev))
				$result[] = (($condition)? '' : 'NOT ') . '(facsev & 7) = ' . $sev[$arg];

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}


class TimeType extends ColType
{
	public static function column($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
		{
			$limits = split(',', $arg);
			$result[] = $column . (($condition)? '' : ' NOT') . " BETWEEN '$limits[0]' AND '$limits[1]'";
		}

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}


	public static function equals($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . '::varchar' . (($condition)? '' : ' NOT') . " LIKE '$arg%'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}


class IntType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			if (ctype_digit($arg))
				$result[] = $column . (($condition)? ' = ' : ' != ') . $arg;

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}


class PIDType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . (($condition)? ' = ' : ' != ') . "'$arg'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}

	public static function tilda($condition, $column, $args)
	{
		$result = array();

		foreach($args as $arg)
			$result[] = $column . (($condition)? ' ' : ' !') . "~ E'" . pg_escape_string($arg) . "'";

		return (count($result))? (' ( ' . implode(($condition)? ' OR ' : ' AND ', $result) . ' ) ') : '';
	}
}


class FullTextType extends ColType
{
	public static function equals($condition, $column, $args)
	{
		$result = array();
		$xchars = array('(', ')', '!', '&', '|', ':', '\\', '\'');

		foreach($args as $arg)
		{
			$arg = trim($arg);
			if ($arg === '')
				continue;
			$arg = str_replace($xchars, '', $arg);
			$arg = preg_replace('/\s\s+/', ' ', $arg);
			$arg = pg_escape_string($arg);
			$arg = '(' . str_replace(' ', ' & ', $arg) . ')';
			$result[] = $arg;
		}

		return (count($result))? ((($condition)? '' : '!') . ' ( ' . implode(' | ', $result) . ' ) ') : '';
	}
}
