var rxPath = /^([-+_.@!~()/:0-9A-Za-z]+)$/;
var rxNumber = /^([1-9])([0-9]{0,2})$/;
var rxHost = /^(http|https):\/\/[1-9a-z][0-9a-z\.-]*[0-9a-z]$/;

function error(message)
{
	alert(message);
	return false;
}

function select_parents(element)
{
	var parent = element.attr('title').slice(9);

	if (parent == 'All')
		return;

	element = $('#filters_out option[value="' + parent + '"]');
	if (element.length)
	{
		element.attr('selected', 'selected');
		select_parents(element);
	}
	// if parent element does not exist - it must be in #filters_in already
}

function select_children(element)
{
	$('#filters_in option[title="' + 'child of ' + element.val() + '"]').each(function(){
		$(this).attr('selected', 'selected');
		select_children($(this));
	});
}

$(document).ready(function(){
	$('#settings-container').load('/logserver/comps/settings', function(){
		$('#changesettingsform').livequery(function(){
			Custom.initstyle();
			$(this).ajaxForm({
				target: '#settings-container',
				beforeSubmit: function(){
					if ($('#db_period').val() == "")
						return error('Please set database rotation period.');

					if ($('#local_period').val() == "")
						return error('Please set local archive rotation period.');

					if ($('#remote_period').val() == "")
						return error('Please set remote archive rotation period.');

					if ($('#local_path').val() == "")
						return error('Please set local path.');

					if ($('#remote_path').val() == $('#local_path').val())
						return error('Remote and local paths cannot be the same.');

					if (!rxNumber.test($('#db_period').val()))
						return error('Database rotation period is invalid.');

					if (!rxNumber.test($('#local_period').val()))
						return error('Local archive rotation period is invalid.');
					
					if (!rxNumber.test($('#remote_period').val()))
						return error('Remote archive rotation period is invalid.');

					if (!rxPath.test($('#local_path').val()))
						return error('Local path is invalid.');

					if ($('#remote_path').val() != '' && !rxPath.test($('#remote_path').val()))
						return error('Remote path is invalid.');

					if ($('#monitor_host').val() != '' && !rxHost.test($('#monitor_host').val()))
						return error('Hostname is invalid.');
				}
			});

			$('input, select').focus(function(){
				$('tr.msg').remove();
			});
		});
	});

	$('.error, .ok').livequery(function(){
		setTimeout("$('.msg').remove()", 5000);
	});

	$('#importfiltersform').ajaxForm({
		target: '#import_status',
		success: function(){
			// jQuery 1.2.x inserts only text (instead of tr & th) in thead
			// so this is another way to add tr.msg
			if ($('#import_status .ok').length)
				$('#importfiltersform table').prepend('<tr class="msg ok">'
					+ '<th colspan="2">' + $('#import_status span').text() + '</th></tr>');
			else
				$('#importfiltersform table').prepend('<tr class="msg error">'
					+ '<th colspan="2">' + $('#import_status span').text() + '</th></tr>');
		}
	});

	$('#upload').change(function(){
		$('#fake_input').val($(this).val());
	});

	$('#import').click(function(){
		if (!$('#upload').val())
			return false;
	});

	$('#filters_out').load('/logserver/comps/filters?action=get_exportable');
	$('#filters_out').focus(function(){
		$('#filters_in').val('');
	});
	$('#filters_in').focus(function(){
		$('#filters_out').val('');
	});
	$('#filters_add').click(function(){
		$('#filters_in').val('');
		$('#filters_out option:selected').each(function(){
			select_parents($(this));
		});
		$('#filters_out option:selected').clone().prependTo('#filters_in');
		$('#filters_out option:selected').remove();
		return false;
	});
	$('#filters_remove').click(function(){
		$('#filters_out').val('');
		$('#filters_in option:selected').each(function(){
			select_children($(this));
		});
		$('#filters_in option:selected').clone().prependTo('#filters_out');
		$('#filters_in option:selected').remove();
		return false;
	});

	// THIS IS IMPORTANT!!!
	// otherwise only selected filters will be passed
	$('#export').click(function(){
		if (!$('#filters_in option').length)
			return false;
		else
			$('#filters_in option').attr('selected', 'selected');
	});
});

(function($) {
	var DataTable = function(node, opts) {
		this.node = $(node);
		this.source_url = this.node.attr("data-source");
		this.data = {};
		this.update();
	};
	DataTable.prototype = {
		update: function() {
			var xhr = $.ajax({
				type: "post",
				data: { action: "getItems" },
				dataType: "json",
				widget: this,
				url: this.source_url,
				success: function(data) {
					this.widget.data = data;
					this.widget.render();
				},
				error: function() {
					console.error(arguments)
				}
			});
		},
		render: function() {
			this.node.find("tbody").empty();
			var html = "";
			$.each(this.data, function(k) {
				html += "<tr>";
				$.each(this, function() {
					html += "<td>"+this+"</td>";
				});
				html += '<td data-name="'+this[0]+'" data-id="'+k+'"><a title="Edit" class="but-action edit"></a><a title="Delete" class="but-action delete"></a></td>';
				html += "</tr>";
			});
			this.node.find("tbody").html(html);
			$(".but-action.delete", this.node).bind("click", function() {
				var id = parseInt($(this).parent().attr("data-id"));
				if (!id) {
					console.error("Invalid id");
					return;
				}
				if (confirm("Would you really like to delete ldap group?")) {
					$.getJSON("/logserver/comps/ldapmap?action=deleteItem&id="+id, function(res) {
						window.location = "/logserver/view/settings";
					});
				}
			});
			$(".but-action.edit", this.node).bind("click", function() {
				var id = parseInt($(this).parent().attr("data-id"));
				var name = $(this).parent().attr("data-name");
				if (!id) {
					console.error("Invalid id");
					return;
				}
				$.ajax({
					url: "/logserver/comps/ldapmap",
					type: "post",
					data: {action: "getInfo", id: id},
					dataType: "json",
					success: function(data) {
						var grp = $("#add-new-map-wrapper");
						grp.find(".ldapmap-actions input[type=hidden]").val("editItem");
						grp.show(300);
						grp.find("#old_id").val(id);
						var out_select;
						var in_select = $("#ldapmap_in");
						(out_select = $("#ldapmap_out")).empty();
						$("#ldapmap_in").empty();
						var out_html = "";
						$.each(window.__group_list_data, function() { 
							out_html += '<option value="'+this.id+'" title="'+this.description+'">'+this.name+'</option>'; 
						});
						$("#ldapmap_out").html(out_html);
						$.each(data, function() {
							in_select.append(out_select.find("option[value="+this.id+"]"));
						});
						
						$(".ldapmap-name input[type=text]").val(name);
						$("#add-new-group-button").hide();
					}
				});
			});
		}
	};

	var def_opts = {};
	$.fn.dataTable = function(options) {
		$(this).each(function() { new DataTable(this, $.extend(def_opts, options)); });
	};
})(jQuery);

$(function() {
	$.getJSON("/logserver/comps/groups?action=get_group_list", function(data) {
		window.__group_list_data = data;
	});
	$(".button-wrapper .to-left").bind("click", function() {
		var items = $("#ldapmap_in option:selected");
		if (items.length) {
			$("#ldapmap_out").append(items);
		}
	});
	$(".button-wrapper .to-right").bind("click", function() {
		var items = $("#ldapmap_out option:selected");
		if (items.length) {
			$("#ldapmap_in").append(items);
		}
	});

	$("#add-new-group-button input[type=button]").bind("click", function() {
		var grp = $("#add-new-map-wrapper");
		grp.find(".ldapmap-actions input[type=hidden]").val("addItem");
		grp.show(300);
		$("#ldapmap_out").empty();
		$("#ldapmap_in").empty();
		var html = "";
		$.each(window.__group_list_data, function() { html += '<option value="'+this.id+'" title="'+this.description+'">'+this.name+'</option>'; });
		$("#ldapmap_out").html(html);
		$(".ldapmap-name input[type=text]").val("");
		$(this).parent().hide();
	});
	$(".ldapmap-actions input[type=button]").bind("click", function() {
		$("#add-new-map-wrapper").hide(300);
		$("#add-new-group-button").show();
	});
	$(".data-table").dataTable();
	$("#add-new-map-wrapper").bind("submit", function() {
		if ($(this).find(".ldapmap-name .name").val().length < 3) {
			alert("Too short name");
			return false;
		}
		$("#ldapmap_in option").attr('selected', 'selected');
		return true;
	});

	$("#use-ldap").bind("change", function() {
		$(this).parent().submit();
	});
});
