#!/usr/bin/php
<?php

/*
 * Author: Arsenij Chystyakov <achystyakov@n-ix.com.ua>
 *
 * Poller create/update RRD for summary reports.
 *
 * */

set_include_path('.:/opt/logserver/gui/');
require ('libs/config.php');
require ('libs/database.php');
require ('comps/inquire/lsqtypes.php');
require ('comps/inquire/lsqparser.php');
require ('comps/inquire/lsq2sql.php');

class Poller
{
	private $rrd_path = "/opt/logserver/rrd/";

	public function __construct()
	{
		$this->getTableMessages();
		$this->checkRRD();
		closelog();
	}

	private function getTableMessages()
	{
		$start_timestamp = time() - 360;
		$end_timestamp = time() - 120;
		$start = date("Y-m-d H:i:00", $start_timestamp);
		$end = date("Y-m-d H:i:59", $end_timestamp);
		$table = date("Y_m_d_H", $start_timestamp); // both timestamps are within the same hour anyway
		Database::query("CREATE TEMP TABLE tmp_tbl AS SELECT * FROM messages_$table"
			. " WHERE rtime >= '$start' AND rtime <= '$end'");
	}

	private function getFilterName()
	{
		$filters_id = array();
		$filters_name = array();
		$filters_query = array();

		$result = Database::query("SELECT filter_id, name, query FROM summary_reports AS s, filters AS f WHERE s.filter_id = f.id");

		foreach($result as $filter)
		{
			$filters_id[] = $filter['filter_id'];
			$filters_name[] = $filter['name'];
			$filters_query[] = $filter['query'];
		}

		$filters = array("id" => $filters_id, "name" => $filters_name, "query" => $filters_query);

		return $filters;
	}

	private function getAllRRD()
	{
		$all_rrd_db = array();

		$dir_handler = opendir($this->rrd_path);
		while(($files = readdir($dir_handler)) !== false)
		{
			if($files != "." && $files != ".." && (substr($files, -4) == ".rrd"))
			{
				$all_rrd_db[] = substr($files, 0, -4);
			}
		}

		return $all_rrd_db;
	}

	private function getLogsNums($query)
	{
		try
		{
			$sql = trim(LSQ2SQL::translate(LSQParser::parseQuery($query)));
		}
		catch (Exception $e)
		{
			return 0;
		}
		//Get logs count
		$result = Database::query("SELECT COUNT(*) FROM tmp_tbl"
			. (($sql !== '')? " WHERE $sql" : ''));

		return $result[0]['count'];
	}

	private function getHostsNums($id, $query)
	{
		$all_hosts = array();
		$host_count = array();
		$now_in_db = array();
		try
		{
			$sql = trim(LSQ2SQL::translate(LSQParser::parseQuery($query)));
		}
		catch (Exception $e)
		{
			return;
		}
		//Get list of hosts/count for filters
		$hosts = Database::query("SELECT host, COUNT(*) AS numb FROM tmp_tbl"
			. (($sql !== '')? " WHERE $sql" : '') . " GROUP BY host");
		//Get all register host for summary report filter
		$register = Database::query("SELECT host FROM summary_reports_hosts AS sh, summary_reports AS sr"
			. " WHERE sh.summary_id=sr.id AND sr.filter_id='$id'");

		foreach($hosts as $host)
		{
			$all_hosts[] = $host['host'];
			$host_count[] = $host['numb'];
		}

		foreach($register as $host)
			$now_in_db[] = $host['host'];

		$need_insert = array_diff($all_hosts, $now_in_db);
		$need_update = array_intersect($all_hosts, $now_in_db);

		if (count($need_insert) > 0)
		{
			$i = 0;
			foreach ($need_insert as $host)
			{
				$sid = Database::query("SELECT id FROM summary_reports WHERE filter_id='$id'");
				Database::query("INSERT INTO summary_reports_hosts (summary_id, host, count)"
					. " VALUES ('{$sid[0]['id']}', '$host', '$host_count[$i]')");
				$i++;
			}
		}

		if (count($need_update) > 0)
		{
			$i = 0;
			foreach ($need_update as $host)
			{
				$sid = Database::query("SELECT id FROM summary_reports WHERE filter_id='$id'");
				Database::query("UPDATE summary_reports_hosts SET count=count+'$host_count[$i]'"
					. " WHERE host='$host' AND summary_id='{$sid[0]['id']}'");
				$i++;
			}
		}
	}

	private function checkRRD()
	{
		$nums = 1;

		$filters = $this->getFilterName();
		$rrd_db = $this->getAllRRD();

		$filter_id_flip = array_flip($filters['id']);

		$need_create = array_diff($filters['id'], $rrd_db);
		$need_update = array_intersect($filters['id'], $rrd_db);
		$need_delete = array_diff($rrd_db, $filters['id']);

		foreach ($need_create as $id)
		{
			$this->createRRD($id);
			$filter_name = $filters['name'][$filter_id_flip[$id]];
			$filter_query = $filters['query'][$filter_id_flip[$id]];
			$nums = $this->getLogsNums($filter_query);
			$this->getHostsNums($id, $filter_query);
			$this->updateRRD($id, $nums);
		}

		foreach ($need_update as $id)
		{
			$filter_name = $filters['name'][$filter_id_flip[$id]];
			$filter_query = $filters['query'][$filter_id_flip[$id]];
			$nums = $this->getLogsNums($filter_query);
			$this->getHostsNums($id, $filter_query);
			$this->updateRRD($id, $nums);
		}

		foreach ($need_delete as $id)
		{
			unlink("$this->rrd_path$id.rrd");
		}
	}

	private function createRRD($id)
	{
		system("/opt/logserver/reports/rrd/create.sh $id");
	}

	private function updateRRD($id, $nums)
	{
		system("/opt/logserver/reports/rrd/update.sh $id $nums");
	}
}

$obj = new Poller();
